//= require <dSS/data/DeviceStore>
//= require <dSS/data/Loader>
//= require <dSS/util/Util>
//= require <dSS/util/DefaultDialog>
//= requie  <dSS/config/Config>
//= require <dSS/devices/DeviceProperties>
//= require <dSS/devices/DeviceSettings.js>

Ext.namespace('dSS', 'dSS.grid');

// custom parameter "featureConfig"
dSS.grid.DevicePanel = Ext.extend(Ext.grid.GridPanel, {
    constructor: function(config) {
        this.featureConfig = config.featureConfig;

        var commentRenderer = function(value, metaData, record, rowIndex, colIndex, store) {
            var comment = record.get('comment');

            if ((comment !== undefined) && (comment.length > 0)) {
                return String.format('<div qtip="{0}">{1}</div>',
                        comment, comment);
            } else {
                return "";
            }
        };

        var dSIDdeviceRenderer= function(value, metaData, record, rowIndex, colIndex, store) {
            var shortId = dSS.util.shortDSID(record.get('dsid'));
            return String.format('<div qtip="{0}">{1}</div>',
                                  record.get('dsid'), shortId);
        };

        var dSIDmeterRenderer= function(value, metaData, record, rowIndex, colIndex, store) {
            var shortId = dSS.util.shortDSID(record.get('meterDSID'));
            return String.format('<div qtip="{0}">{1}</div>',
                                  record.get('meterDSID'), shortId);
        };

        var devTypeRenderer = function(value, metaData, record, rowIndex, colIndex, store) {

            var present = record.get('isPresent');

            var fid = dSS.util.parseFunctionID(record.get('functionID'));
            var prod = dSS.util.parseProductID(record.get('productID'));

            var retval = String.format('<img class="padding-img" src="{0}"/>', Ext.BLANK_IMAGE_URL);
            var cls = dSS.util.getDeviceIconCSS(prod[0], fid[0], false);
            if (cls !== undefined) {
                metaData.css = cls;
            } else {
                retval = "";
            }
            return retval;
        };

        var zoneNameRenderer = function(value, metaData, record, rowIndex, colIndex, store) {
            return value ? value: dSS._("Zone") + ' #' + record.get('zoneId');
        };

        var deviceCols = [
        {
            header: "",
            width: 20,
            resizable: false,
            sortable: false,
            hideable: false,
            menuDisabled: true,
            fixed: true,
            dataIndex: 'functionID',
            renderer: devTypeRenderer,
            id: 'functionID'
        },
        {
            header: dSS._("name"),
            tooltip: dSS._("name"),
            width: 80,
            sortable: true,
            dataIndex: 'name',
            id: 'name'
        },
        {
            header: dSS._("dSID"),
            tooltip: dSS._("dSID"),
            width: 70,
            sortable: true,
            dataIndex: 'dsid',
            id: 'dsid',
            renderer: dSIDdeviceRenderer
        }];

        if (this.featureConfig !== dSS.config.FeatureConfig.CIRCUIT) {
            deviceCols.push(
            {
                header: dSS._("dSM Name"),
                tooltip: dSS._("dSM Name"),
                width: 80,
                sortable: true,
                dataIndex: 'modulatorName',
                id: 'modulatorName'
            });
        }

        deviceCols.push(
        {
            header: dSS._("Room name"),
            tooltip: dSS._("Room name"),
            width: 80,
            sortable: true,
            dataIndex: 'zoneName',
            id: 'zoneName',
            renderer: zoneNameRenderer
        });

        if (this.featureConfig === dSS.config.FeatureConfig.CIRCUIT) {
            deviceCols = deviceCols.concat(
            [
                {
                    header: dSS._("Room ID"),
                    tooltip: dSS._("Room ID"),
                    width: 35,
                    sortable: true,
                    dataIndex: 'zoneId',
                    id: 'zoneId'
                },
                {
                    header: dSS._("HW info"),
                    tooltip: dSS._("Hardware info"),
                    width: 50,
                    sortable: true,
                    id: 'HW',
                    dataIndex: 'HW'
                },
                {
                    header: dSS._("Firmware version"),
                    tooltip: dSS._("Firmware version"),
                    width: 50,
                    sortable: true,
                    id: 'fw',
                    dataIndex: 'firmwareVersion'
                },
                {
                    header: dSS._("Registered on"),
                    tooltip: dSS._("Registered on"),
                    width: 60,
                    sortable: true,
                    dataIndex: 'firstSeen',
                    xtype: 'datecolumn',
                    format: 'Y-m-d\\&\\n\\b\\s\\p\\;H:m:i',
                    id: 'firstSeen'
                },
                {
                    header: dSS._("Last seen on"),
                    tooltip: dSS._("Last seen on"),
                    width: 60,
                    sortable: true,
                    dataIndex: 'lastDiscovered',
                    xtype: 'datecolumn',
                    format: 'Y-m-d\\&\\n\\b\\s\\p\\;H:m:i',
                    id: 'lastDiscovered'
                }
            ]);
        }

        if ((this.featureConfig === dSS.config.FeatureConfig.ZONE) ||
            (this.featureConfig === dSS.config.FeatureConfig.CIRCUIT)) {
            deviceCols.push(
                {
                    header: dSS._("Comment"),
                    tooltip: dSS._("Comment"),
                    width: 20,
                    resizable: false,
                    sortable: false,
                    dataIndex: 'comment',
                    id: 'comment',
                    renderer: commentRenderer
                });
        }

        var selModel;

        if ((this.featureConfig === dSS.config.FeatureConfig.SCENE) ||
            (this.featureConfig === dSS.config.FeatureConfig.CIRCUIT)) {
            selModel = new Ext.grid.RowSelectionModel({ singleSelect: true });
        } else {
            selModel = new Ext.grid.RowSelectionModel({ singleSelect: false });
        }

        var enabledd = false;
        if (this.featureConfig === dSS.config.FeatureConfig.ZONE) {
            enabledd = true;
        }

        Ext.apply(this, {
            store: dSS.data.Loader.getDeviceStore(),
            columns: deviceCols,
            ddGroup: "zoneDeviceDD",
            enableDragDrop: enabledd,
            stripeRows: true,
            title: dSS._("Device", 2),
            selModel: selModel,
            viewConfig: {
                forceFit: true,
                getRowClass: function(record, index) {
                    var c = record.get('isPresent');
                    if (c === false) {
                        return 'nonPresentDevice';
                    }
                    return '';
                }
            }
        });

        this.tbar = this.buildTopToolbar();

        dSS.grid.DevicePanel.superclass.constructor.apply(this, arguments);

        // Here you can add functionality that requires the object to
        // exist, like event handling.
        this.on('rowcontextmenu', function(grid, rowIndex, event) {
            event.preventDefault();
            event.stopEvent();
            if (!this.contextMenu) {
                this.contextMenu = new Ext.menu.Menu({});
            } else {
                this.contextMenu.removeAll();
            }

            var sel = this.getSelectionModel();
            if (!sel.isSelected(rowIndex)) {
                sel.selectRow(rowIndex, false);
            }

            var menuItem;
            var showRemoveMenuItem = false;
            var presentInSelection = false;
            var selectedDeviceIsPresent = false;

            if (sel.getSelected().get("isPresent") === true) {
                selectedDeviceIsPresent = true;
            }

            var records = sel.getSelections();

            var i;
            for (i = 0; i < records.length; i++) {
                var record = records[i];

                // if at least one record can be removed - set the flag
                if (record.get('isPresent') === false) {
                    showRemoveMenuItem = true;
                } else {
                    presentInSelection = true;
                }

                if ((showRemoveMenuItem === true) && (presentInSelection === true)) {
                    break;
                }
            }

            menuItem = new Ext.menu.Item({
                text: dSS._("Blink"),
                iconCls: 'blink',
                handler: this.blinkDevice.createDelegate(this, rowIndex,
                                                         true),
                disabled : !presentInSelection
            });
            this.contextMenu.add(menuItem);

            menuItem = new Ext.menu.Item({
                text: dSS._("Edit device values"),
                iconCls: 'editValueToolbarIcon',
                handler: this.showControlDeviceWindow.createDelegate(
                                                this, rowIndex, true),
                disabled: !selectedDeviceIsPresent
            });
            this.contextMenu.add(menuItem);

            this.contextMenu.add('-');

            if (this.featureConfig === dSS.config.FeatureConfig.CIRCUIT) {
                menuItem = new Ext.menu.Item({
                    text: dSS._("Test transmission quality"),
                    iconCls: 'transmissionQuality',
                    handler: this.getTransmissionQuality.createDelegate(
                                                    this, rowIndex, true),
                    disabled: !selectedDeviceIsPresent
                });
                this.contextMenu.add(menuItem);

                this.contextMenu.add('-');
            }

            if ((this.featureConfig === dSS.config.FeatureConfig.SCENE) ||
                (this.featureConfig === dSS.config.FeatureConfig.CIRCUIT)) {
                menuItem = new Ext.menu.Item({
                    text: dSS._("Edit device properties"),
                    iconCls: (this.featureConfig === dSS.config.FeatureConfig.SCENE) ? 'sceneDevicePropertiesIcon' : 'devicePropertiesIcon',
                    handler: this.showDevicePropertiesWindow.createDelegate(
                                                    this, rowIndex, true),
                    disabled: !selectedDeviceIsPresent
                });
                this.contextMenu.add(menuItem);
            }

            if ((this.featureConfig === dSS.config.FeatureConfig.ZONE) ||
                (this.featureConfig === dSS.config.FeatureConfig.CIRCUIT)) {
                menuItem = new Ext.menu.Item({
                    text: dSS._("Rename device"),
                    iconCls: 'editIcon',
                    handler: this.editDevice.createDelegate(this, rowIndex, true)
                });
                this.contextMenu.add(menuItem);

                menuItem = new Ext.menu.Item({
                    text: dSS._("Edit comment"),
                    iconCls: 'editCommentIcon',
                    handler: this.editComment.createDelegate(this, rowIndex, true)
                });
                this.contextMenu.add(menuItem);
            }

            this.contextMenu.add('-');

            if ((this.featureConfig === dSS.config.FeatureConfig.ZONE) ||
                (this.featureConfig === dSS.config.FeatureConfig.CIRCUIT)) {

                menuItem = new Ext.menu.Item({
                    text: dSS._("Remove device"),
                    iconCls: 'removeDeviceIcon',
                    handler: this.removeDevice.createDelegate(this, rowIndex,
                                                              true),
                    disabled: !showRemoveMenuItem
                });
                this.contextMenu.add(menuItem);
            }
            var xy = event.getXY();
            this.contextMenu.showAt(xy);
        },
        this);
    },

    buildTopToolbar: function() {
        var bar = [];

        bar.push({
            iconCls: 'blink',
            handler: this.blinkDevice,
            tooltip: dSS._("Blink device"),
            scope: this
        });

        bar.push(
            {
                iconCls: 'editValueToolbarIcon',
                handler: this.showControlDeviceWindow,
                tooltip: dSS._("Edit device values"),
                scope: this
            });

        bar.push('-');

        if (this.featureConfig === dSS.config.FeatureConfig.CIRCUIT) {
            bar = bar.concat(
            [
                {
                    iconCls: 'transmissionQuality',
                    handler: this.getTransmissionQuality,
                    tooltip: dSS._("Test transmission quality"),
                    scope: this
                },
                '-'
           ]);
        }

        if ((this.featureConfig === dSS.config.FeatureConfig.SCENE) ||
            (this.featureConfig === dSS.config.FeatureConfig.CIRCUIT)) {
            bar.push(
                {
                    iconCls: (this.featureConfig === dSS.config.FeatureConfig.SCENE) ? 'sceneDevicePropertiesIcon' : 'devicePropertiesIcon',
                    handler: this.showDevicePropertiesWindow,
                    tooltip: dSS._("Configure device properties"),
                    scope: this
                });
        }

        if ((this.featureConfig === dSS.config.FeatureConfig.ZONE) || 
            (this.featureConfig === dSS.config.FeatureConfig.CIRCUIT)) {
            bar = bar.concat([
            {
                iconCls: 'editDeviceToolbarIcon',
                handler: this.editDevice,
                tooltip: dSS._("Rename device"),
                scope: this
            },
            {
                iconCls: 'editCommentToolbarIcon',
                handler: this.editComment,
                tooltip: dSS._("Edit comment"),
                scope: this
            }]);
        }

        bar.push('->');

        if ((this.featureConfig === dSS.config.FeatureConfig.ZONE) ||
            (this.featureConfig === dSS.config.FeatureConfig.CIRCUIT)) {
            bar.push(
            {
                iconCls: 'removeDeviceToolbarIcon',
                handler: this.removeDevice,
                tooltip: dSS._("Remove device"),
                scope: this
            });
        }

        return bar;
    },

    showControlDeviceWindow: function(item, event, rowIndex) {
        var records = this.getStore().getRange();
        var sel = this.getSelectionModel();
        var index;

        if (rowIndex === undefined) {
            // we were called from the toolbar, so its possible that there is
            // no device selection yet
            if (!sel.hasSelection()) {
                Ext.Msg.alert(dSS._("Error"),
                    dSS._("No device selected, please select a device from the list!"));
                return;
            }

            var dsid = sel.getSelected().get('dsid');
            var i;
            for (i = 0; i < records.length; i++) {
                if (records[i].get('dsid') === dsid) {
                    index = i;
                    break;
                }
            }
        } else {
            index = rowIndex;
        }

        var win = new dSS.devices.DeviceSettingsWindow({
            devices: records,
            index: index,
            rowSel: sel,
            maskElement: this.findParentByType(Ext.Viewport).el
        });
        win.show(this);
    },
    editDevice: function(item, event, rowIndex) {
        var record;

        if (rowIndex === undefined) {
            // we were called from the toolbar, so its possible that there is
            // no device selection yet
            var sel = this.getSelectionModel();
            if (!sel.hasSelection()) {
                Ext.Msg.alert(dSS._("Error"),
                    dSS._("No device selected, please select a device from the list!"));
                return;
            }

            record = sel.getSelected();
        } else {
            record = this.getStore().getAt(rowIndex);
        }

        var oldName = record.get('name');

        Ext.Msg.prompt(dSS._("Rename device"), dSS._("Name for") + ' ' + dSS.util.shortDSID(record.get('dsid')) + ': ', function(btn, text) {
            if (text !== record.get('name')) {
                Ext.Ajax.request({
                    url: '/json/device/setName',
                    disableCaching: true,
                    method: "GET",
                    params: {
                        dsid: record.get('dsid'),
                        newName: text
                    },
                    success: function(result, request) {
                        try {
                            var jsonData = Ext.util.JSON.decode(result.responseText);
                            if (jsonData.ok) {
                                record.set('name', text);
                                record.commit();
                                // we are only interested in name changes
                                // from nothing to something or vice versa
                                //
                                // check that the names were not equal is done
                                // before submitting the ajax request
                                if ((oldName === undefined) ||
                                    (oldName.length === 0) ||
                                    (text === undefined) ||
                                    (text.length === 0)) {
                                    dSS.data.Loader.getZoneStore().fireEvent(
                                            'zonedevnamechanged', record);
                                }
                                dSS.MessageBus.fireEvent('statusmessage',
                                        dSS._("Device") + ' &quot;' +
                                        dSS.util.getName(record.get('name')) +
                                        '&quot; (' +
                                        dSS.util.shortDSID(record.get('dsid')) +
                                        ') ' + dSS._("was renamed"));
                            } else {
                                Ext.Msg.alert(dSS._("Error"),
                                              dSS._("Could not rename device") +
                                              ': ' + jsonData.message);
                            }
                        }
                        catch(err) {
                            Ext.Msg.alert(dSS._("Error"),
                                          dSS._("Could not rename device") +
                                          ': ' + err);
                        }
                    },
                    failure: function(result, request) {
                        Ext.Msg.alert(dSS._("Error"),
                                      dSS._("Could not rename device"));
                    }
                });
            }
        },
        this, false, record.get('name'));
    },

    getTransmissionQuality: function(item, event, rowIndex) {
        var record;

        if (rowIndex === undefined) {
            // we were called from the toolbar, so its possible that there is
            // no device selection yet
            var sel = this.getSelectionModel();
            if (!sel.hasSelection()) {
                Ext.Msg.alert(dSS._("Error"),
                    dSS._("No device selected, please select a device from the list!"));
                return;
            }

            record = sel.getSelected();
        } else {
            record = this.getStore().getAt(rowIndex);
        }

        if (record.get('isPresent') === false) {
            Ext.Msg.alert(dSS._("Error"), dSS._("Device not present: can not request transmission quality from a non present device!"));
            return;
        }

        var name = record.get('name');
        var dsid = record.get('dsid');

        Ext.Ajax.request({
            scope: this,
            url: '/json/device/getTransmissionQuality',
            disableCaching: true,
            method: "GET",
            params: {
                dsid: dsid
            },
            success: function(result, request) {
                try {
                    var jsonData = Ext.util.JSON.decode(result.responseText);
                    if (jsonData.ok) {
                        var title = dSS._("Transmission Quality");

                        var up = jsonData.result.upstream * (100/dSS.config.TransmissionQuality.UPSTREAM_MAX);
                        var down = 100 - jsonData.result.downstream * (100/dSS.config.TransmissionQuality.DOWNSTREAM_MAX);
                        up = Math.round(up);
                        down = Math.round(down);

                        var win = new dSS.util.DefaultDialogWindow({
                            title: title,
                            name: name,
                            dsid: dsid,
                            extraItems:
                            [
                                {
                                    xtype: 'dssutildialogcontentpanel',
                                    strings:
                                    [
                                        dSS._("Upstream communication")+':&nbsp;',
                                        up + '%',
                                        dSS._("Downstream communication")+':&nbsp;',
                                        down + '%'
                                    ]
                                }
                            ],
                            maskElement: this.findParentByType(Ext.Viewport).el
                        });
                        win.show();

                    } else {
                        Ext.MessageBox.alert(dSS._("Error"),
                                dSS._("Could not get transmission quality: "),
                                  jsonData.message);
                    }
                }
                catch(err) {
                    Ext.Msg.alert(dSS._("Error"),
                                  dSS._("Could not get transmission quality") +
                                  ': ' + err);
                }
            },
            failure: function(result, request) {
                var message = '';
                try {
                    var jsonData = Ext.util.JSON.decode(result.responseText);
                    if ((!jsonData.ok) && (jsonData.message.length > 0)) {
                        message = ': ' + jsonData.message;
                    }
                }
                catch(err) {}
                Ext.MessageBox.alert(dSS._("Error"),
                        dSS._("Could not get transmission quality") + message);
            }
        });
    },

    removeDevice: function(item, event, rowIndex) {
        var records;

        // we were called from the toolbar, so its possible that there is
        // no device selection yet
        var sel = this.getSelectionModel();
        if (!sel.hasSelection()) {
            Ext.Msg.alert(dSS._("Error"),
                dSS._("No device selected, please select a device from the list!"));
            return;
        }
        records = sel.getSelections();

        var skippedPresent = false;
        var notPresent = [];

        var i;
        for (i = 0; i < records.length; i++) {
            var record = records[i];

            // do not issue the ajax request if we know that it will fail
            if (record.get('isPresent') === true) {
                skippedPresent = true;
                continue;
            }

            notPresent.push(record);
        }

        if (notPresent.length > 0) {
            this.deleteDeviceConfirm(notPresent);
        } else {
            if (skippedPresent === true) {
                Ext.MessageBox.alert(dSS._("Error"), dSS._("Present devices can not be removed!"));
            }
        }
    },

    deleteDeviceConfirm: function(records) {
        var store = this.getStore();
        if (records.length < 1) {
            this.findParentByType(Ext.Viewport).el.unmask();
            return;
        }

        var record = records.shift();

        var win = new dSS.util.DefaultDialogWindow({
            title: dSS._("Delete Device"),
            name: record.get('name'),
            dsid: record.get('dsid'),
            cancelButtonText: dSS._("Cancel"),
            scope: this,
            extraItems:
            [
                {
                    xtype: 'panel',
                    html: '<p>' +
                          dSS._("Are you sure you want to delete this " +
                                "device?") + '</p><p>' +
                          dSS._("Deleting the device will remove its " +
                                "configuration. Therefore, the device should " +
                                "only be deleted if it is permanently " +
                                "removed from this digitalSTROM " +
                                "installation. If you want to connect the " +
                                "device again, you might need to reregister " +
                                "the device manually within the power " +
                                "circuits tab.") +
                          '</p>'
                }
            ],
            maskElement: this.findParentByType(Ext.Viewport).el,
            buttons:
            [
                {
                    xtype: 'button',
                    text: dSS._("Delete"),
                    scope: this,
                    handler: function() {
                        win.close();
                        var f = this.deleteDevice.createDelegate(this,
                                                                 [
                                                                     record,
                                                                     records
                                                                 ]);
                        f();
                    }
                }
            ]
        });
        win.show();
    },

    deleteDevice: function(record, remainingRecords) {
        maskel = this.findParentByType(Ext.Viewport).el;
        maskel.mask(dSS._("Deleting device") + ": " + record.get("dsid"));
        Ext.Ajax.request({
            url: '/json/structure/removeDevice',
            disableCaching: true,
            method: "GET",
            scope: this,
            params: {
                deviceID: record.get('dsid')
            },
            success: function(result, request) {
                try {
                    var jsonData = Ext.util.JSON.decode(result.responseText);
                    if (jsonData.ok) {
                        var dSMStore = dSS.data.Loader.getModulatorStore();
                        var i = dSMStore.find('dsid', record.get('meterDSID'));
                        if (i > -1) {
                            var dsm = dSMStore.getAt(i);
                            dsm.set('numDevices', dsm.get('numDevices') - 1);
                            dsm.commit();
                        }
                        this.getStore().remove(record);
                        dSS.MessageBus.fireEvent('statusmessage',
                            dSS._("Device") + ' &quot;' +
                            dSS.util.getName(record.get('name')) + '&quot; (' +
                            dSS.util.shortDSID(record.get('dsid')) +
                            ') ' + _("was removed"));

                        this.deleteDeviceConfirm(remainingRecords);
                    } else {
                        maskel.unmask();
                        Ext.MessageBox.alert(dSS._("Could not remove"), jsonData.message);
                    }
                }
                catch(err) {
                    maskel.unmask();
                    Ext.MessageBox.alert(dSS._("Error"), dSS._("Could send remove device command to dSS") + ": " + err);
                }
            },
            failure: function(result, request) {
                maskel.unmask();
                Ext.MessageBox.alert(dSS._("Error"), dSS._("Could send remove device command to dSS"));
            }
        });
    },

    sendBlinkRequest: function(record) {
        record.set('blink', true);
        var unblink = function(record) {
            record.set('blink', false);
        };

        Ext.Ajax.request({
            url: '/json/device/blink',
            disableCaching: true,
            method: "GET",
            scope: this,
            params: {
                dsid: record.get('dsid')
            },
            success: function(result, request) {
                try {
                    var jsonData = Ext.util.JSON.decode(result.responseText);
                    if (!jsonData.ok) {
                        Ext.Msg.alert(dSS._("Could not blink"),
                                      jsonData.message);
                    } else {
                        dSS.MessageBus.fireEvent('statusmessage',
                            dSS._("Device") + ' &quot;' +
                            dSS.util.getName(record.get('name')) + '&quot; (' +
                            dSS.util.shortDSID(record.get('dsid')) +
                              ') ' + dSS._("is blinking now"));
                    }
                } catch(err) {
                    Ext.MessageBox.alert(dSS._("Error"), dSS._("Could not send blink device command to dSS") + ": " + err);
                }
                unblink.defer(2000, this, [record]);
            },
            failure: function(result, request) {
                Ext.MessageBox.alert(dSS._("Error"), dSS._("Could not send blink device command to dSS"));
                unblink.defer(2000, this, [record]);
            }
        });
    },

    blinkDevice: function(item, event, rowIndex) {
        var records;

        // we were called from the toolbar, so its possible that there is
        // no device selection yet
        var sel = this.getSelectionModel();
        if (!sel.hasSelection()) {
            Ext.Msg.alert(dSS._("Error"),
                dSS._("No device selected, please select a device from the list!"));
            return;
        }
        records = sel.getSelections();
        var i;
        for (i = 0; i < records.length; i++) {
            if (records[i].get('isPresent') === false) {
                continue;
            }

            this.sendBlinkRequest(records[i]);
        }
    },

    showDevicePropertiesWindow: function(item, event, rowIndex) {
        var record;

        if (rowIndex === undefined) {
            // we were called from the toolbar, so its possible that there is
            // no device selection yet
            var sel = this.getSelectionModel();
            if (!sel.hasSelection()) {
                Ext.Msg.alert(dSS._("Error"),
                    dSS._("No device selected, please select a device from the list!"));
                return;
            }

            record = sel.getSelected();
        } else {
            record = this.getStore().getAt(rowIndex);
        }

        if (record.get('isPresent') === false) {
            Ext.Msg.alert(dSS._("Error"), ("Device not present: The selected device was not found and can not be configured!"));
            return;
        }

        var devInfo = dSS.util.parseProductID(record.get('productID'));
        var type = devInfo[0];
        var number = devInfo[1];
        var cls = dSS.util.parseFunctionID(record.get('functionID'));
        var forms = [];
        var showDeviceInfo = true;
        var matrix;
        var extraTitle;

        try {
            matrix = dSS.devprops.GetVisibleProperties(cls[0], type, number);
        } catch (error) {
            // for now we can safely assume all as invisible
            matrix = error;
        }

        if ((this.featureConfig === dSS.config.FeatureConfig.CIRCUIT) &&
            (matrix.outmode === true)) {
            forms.push({
                            xtype: 'dssdevoutputmodeform',
                            dsid : record.get('dsid')
                       });
        }

        if (this.featureConfig === dSS.config.FeatureConfig.SCENE) {
            var scene;
            try {
                scene = this.findParentByType('dssscenespanel').getSelectedScene();
            } catch (err) {
                Ext.Msg.alert(dSS._("Error"), err);
                return;
            }

            var sceneStore = dSS.data.Loader.getSceneStore();
            var sceneId = record.get('zoneId') + '-' + cls[0] + '-' + scene;
            var sceneRec = sceneStore.getById(sceneId);
            var sceneName = "";
            if (sceneRec !== undefined) {
                sceneName = sceneRec.get('name');
            }

            if (sceneName.length > 0) {
                extraTitle = ' - ' + sceneName;
            } else {
                extraTitle = ' - ' + dSS._(dSS.config.sceneNames[scene]);
            }

            if (dSS.devprops.CheckShowSceneProperties(matrix) === true) {
                // info handled by form
                showDeviceInfo = false;
                forms.push({
                            xtype: 'dssdevscenepropsform',
                            dsid : record.get('dsid'),
                            deviceName: record.get('name'),
                            scene: scene,
                            visibilityMatrix: matrix
                       });
            }
        }

        var icon = dSS.util.getDeviceIconCSS(type, cls[0], true);
        var win = new dSS.devprops.DevicePropertiesWindow({
            name: record.get('name'),
            dsid: record.get('dsid'),
            standardForms: forms,
            extraTitle: extraTitle,
            iconCls: icon,
            showDefaults: showDeviceInfo,
            maskElement: this.findParentByType(Ext.Viewport).el
        });
        win.show();
    },

    showCopyWindow: function() {
        var tsvwindow = new dSS.util.GridToSVWindow({
            ignore: [ 'functionID' ],
            gridPanel: this,
            windowWidth: 640,
            windowHeight: 480,
            separator: "\t"
        });
        tsvwindow.show(this);
    },

    editComment: function(item, event, rowIndex) {
        var record;

        if (rowIndex === undefined) {
            // we were called from the toolbar, so its possible that there is
            // no device selection yet
            var sel = this.getSelectionModel();
            if (!sel.hasSelection()) {
                Ext.Msg.alert(dSS._("Error"),
                    dSS._("No device selected, please select a device from the list!"));
                return;
            }

            record = sel.getSelected();
        } else {
            record = this.getStore().getAt(rowIndex);
        }

        Ext.Msg.prompt(
            dSS._("Edit comment"), 
            dSS._("Comment for") + ' ' + dSS.util.shortDSID(record.get('dsid')) + ': ', 
            function(btn, text) {
            if (text !== record.get('comment')) {
                Ext.Ajax.request({
                    url: '/json/property/setString',
                    disableCaching: true,
                    method: "GET",
                    params: {
                        path:   '/apartment/zones/zone0/' +
                                record.get('dsid') + '/comment',
                        value: text
                    },
                    success: function(result, request) {
                        try {
                            var jsonData = Ext.util.JSON.decode(
                                                        result.responseText);
                            if (jsonData.ok) {
                                Ext.Ajax.request({
                                    url: '/json/property/setFlag',
                                    disableCaching: true,
                                    method: "GET",
                                    params: {
                                        flag: 'ARCHIVE',
                                        path:   '/apartment/zones/zone0/' +
                                                record.get('dsid') + '/comment',
                                        value: true
                                    },
                                    success: function(result, request) {
                                        try {
                                            var jsonData = Ext.util.JSON.decode(
                                                result.responseText);
                                            if (jsonData.ok) {
                                                record.set('comment', text);
                                                record.commit();
                                            }
                                        } catch(err) {
                                            Ext.MessageBox.alert(dSS._("Error"), 
                                            dSS._("Could not save comment on dSS") +
                                              ": " + err);
                                        }
                                    },
                                    failure: function(result, request) {
                                        Ext.MessageBox.alert(dSS._("Error"), 
                                            dSS._("Could not save comment on dSS"));
                                    }
                                });
                            } else {
                                Ext.MessageBox.alert(dSS._("Error"), 
                                                    dSS._("Could not add comment"));
                            }
                        }
                        catch(err) {
                            Ext.MessageBox.alert(dSS._("Error"), 
                                     dSS._("Could not add comment") + ": " + err);
                        }
                    },
                    failure: function(result, request) {
                        Ext.MessageBox.alert(dSS._("Error"), 
                                dSS._("Could not send command to dSS"));
                    }
                });
            }
        },
        this, true, record.get('comment'));
    }
});

Ext.reg('dssdevicepanel', dSS.grid.DevicePanel);

